@@ -5,7 +5,7 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: addr.h,v 1.5 2001/12/29 21:05:46 dugsong Exp $
+ * $Id: addr.h,v 1.8 2002/01/20 21:21:00 dugsong Exp $
  */
 
 #ifndef DNET_ADDR_H
@@ -15,15 +15,15 @@
 #define	ADDR_TYPE_IP		2	/* Internet Protocol v4 */
 
 struct addr {
-	u_short			addr_type;
-	u_short			addr_bits;
+	uint16_t		addr_type;
+	uint16_t		addr_bits;
 	union {
 		eth_addr_t	__eth;
 		ip_addr_t	__ip;
 		
-		u_int8_t	__data8[20];
-		u_int16_t	__data16[10];
-		u_int32_t	__data32[5];
+		uint8_t		__data8[20];
+		uint16_t	__data16[10];
+		uint32_t	__data32[5];
 	} __addr_u;
 };
 #define addr_eth	__addr_u.__eth
@@ -32,30 +32,30 @@ struct addr {
 #define addr_data16	__addr_u.__data16
 #define addr_data32	__addr_u.__data32
 
-__BEGIN_DECLS
-int	 addr_cmp(struct addr *a, struct addr *b);
-int	 addr_bcast(struct addr *a, struct addr *b);
-
-int	 addr_ntop(struct addr *src, char *dst, size_t size);
-int	 addr_pton(char *src, struct addr *dst);
-
-char	*addr_ntoa(struct addr *a);
-#define	 addr_aton	addr_pton
-
-int	 addr_ntos(struct addr *a, struct sockaddr *sa);
-int	 addr_ston(struct sockaddr *sa, struct addr *a);
-
-int	 addr_btos(u_short bits, struct sockaddr *sa);
-int	 addr_stob(struct sockaddr *sa, u_short *bits);
-
-int	 addr_btom(u_short bits, void *mask, size_t size);
-int	 addr_mtob(void *mask, size_t size, u_short *bits);
-
 #define addr_fill(addr, type, bits, data, len) do {	\
 	(addr)->addr_type = type;			\
 	(addr)->addr_bits = bits;			\
 	memmove((addr)->addr_data8, (char *)data, len);	\
 } while (0)
+
+__BEGIN_DECLS
+int	 addr_cmp(const struct addr *a, const struct addr *b);
+int	 addr_bcast(const struct addr *a, struct addr *b);
+
+int	 addr_ntop(const struct addr *src, char *dst, size_t size);
+int	 addr_pton(const char *src, struct addr *dst);
+
+char	*addr_ntoa(const struct addr *a);
+#define	 addr_aton	addr_pton
+
+int	 addr_ntos(const struct addr *a, struct sockaddr *sa);
+int	 addr_ston(const struct sockaddr *sa, struct addr *a);
+
+int	 addr_btos(uint16_t bits, struct sockaddr *sa);
+int	 addr_stob(const struct sockaddr *sa, uint16_t *bits);
+
+int	 addr_btom(uint16_t bits, void *mask, size_t size);
+int	 addr_mtob(const void *mask, size_t size, uint16_t *bits);
 __END_DECLS
 
 #endif /* DNET_ADDR_H */
