@@ -5,34 +5,37 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: addr.c,v 1.10 2001/12/29 21:05:46 dugsong Exp $
+ * $Id: addr.c,v 1.15 2002/01/17 21:08:04 dugsong Exp $
  */
 
 #include "config.h"
 
 #include <sys/types.h>
-#include <sys/socket.h>
 
-#include <net/if.h>
+#ifdef HAVE_NET_IF_H
+# include <sys/socket.h>
+# include <net/if.h>
+#endif
 #ifdef HAVE_NET_IF_DL_H
-#include <net/if_dl.h>
+# include <net/if_dl.h>
 #endif
 #ifdef HAVE_NET_RAW_H
-#include <net/raw.h>
+# include <net/raw.h>
 #endif
-#include <netinet/in.h>
-#include <arpa/inet.h>
 
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
-#include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "dnet.h"
 
+#ifndef MAXHOSTNAMELEN
+# define MAXHOSTNAMELEN	256
+#endif
+
 static const char *octet2dec[] = {
 	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12",
 	"13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23",
@@ -91,7 +94,7 @@ static const char *octet2hex[] = {
 };
 
 int
-addr_cmp(struct addr *a, struct addr *b)
+addr_cmp(const struct addr *a, const struct addr *b)
 {
 	int i, j, k;
 	
@@ -118,7 +121,7 @@ addr_cmp(struct addr *a, struct addr *b)
 }
 
 int
-addr_bcast(struct addr *a, struct addr *b)
+addr_bcast(const struct addr *a, struct addr *b)
 {
 	struct addr mask;
 	
@@ -140,10 +143,10 @@ addr_bcast(struct addr *a, struct addr *b)
 }
 
 int
-addr_ntop(struct addr *src, char *dst, size_t size)
+addr_ntop(const struct addr *src, char *dst, size_t size)
 {
 	const char *p;
-	u_char *u;
+	const u_char *u;
 	
 	u = src->addr_data8;
 	
@@ -183,8 +186,30 @@ addr_ntop(struct addr *src, char *dst, size_t size)
 	return (-1);
 }
 
+#ifdef WIN32
+/* XXX - it's total trash, and it's a natural fact that i'm not no cow */
+void
+_close_winsock(void)
+{
+	WSACleanup();
+}
+
+void
+_init_winsock(void)
+{
+	static int initialized;
+	WSADATA wsdata;
+	
+	if (!initialized) {
+		if (WSAStartup(MAKEWORD(2, 2), &wsdata) != 0)
+			return;
+		atexit(_close_winsock);
+	}
+}
+#endif
+
 int
-addr_pton(char *src, struct addr *dst)
+addr_pton(const char *src, struct addr *dst)
 {
 	char *p, tmp[MAXHOSTNAMELEN];
 	long l;
@@ -234,9 +259,11 @@ addr_pton(char *src, struct addr *dst)
 			dst->addr_bits = IP_ADDR_BITS;
 		}
 		if (inet_pton(AF_INET, tmp, &dst->addr_ip) != 1) {
-			struct hostent *hp = gethostbyname(tmp);
-			
-			if (hp == NULL) {
+			struct hostent *hp;
+#ifdef WIN32
+			_init_winsock();
+#endif
+			if ((hp = gethostbyname(tmp)) == NULL) {
 				errno = EINVAL;
 				return (-1);
 			}
@@ -247,12 +274,12 @@ addr_pton(char *src, struct addr *dst)
 }
 
 char *
-addr_ntoa(struct addr *a)
+addr_ntoa(const struct addr *a)
 {
 	static char *p, buf[BUFSIZ];
 	char *q;
 	
-	if (p == NULL || p > buf + sizeof(buf))
+	if (p == NULL || p > buf + sizeof(buf) - 32 /* XXX */)
 		p = buf;
 	
 	if (addr_ntop(a, p, (buf + sizeof(buf)) - p) < 0)
@@ -265,7 +292,7 @@ addr_ntoa(struct addr *a)
 }
 
 int
-addr_ntos(struct addr *a, struct sockaddr *sa)
+addr_ntos(const struct addr *a, struct sockaddr *sa)
 {
 	switch (a->addr_type) {
 	case ADDR_TYPE_ETH:
@@ -300,14 +327,14 @@ addr_ntos(struct addr *a, struct sockaddr *sa)
 		break;
 	}
 	default:
-		errno = EPFNOSUPPORT;
+		errno = EINVAL;
 		return (-1);
 	}
 	return (0);
 }
 
 int
-addr_ston(struct sockaddr *sa, struct addr *a)
+addr_ston(const struct sockaddr *sa, struct addr *a)
 {
 	memset(a, 0, sizeof(*a));
 	
@@ -352,14 +379,14 @@ addr_ston(struct sockaddr *sa, struct addr *a)
 		break;
 	}
 	default:
-		errno = EPFNOSUPPORT;
+		errno = EINVAL;
 		return (-1);
 	}
 	return (0);
 }
 
 int
-addr_btos(u_short bits, struct sockaddr *sa)
+addr_btos(uint16_t bits, struct sockaddr *sa)
 {
 	struct sockaddr_in *sin;
 	
@@ -375,11 +402,11 @@ addr_btos(u_short bits, struct sockaddr *sa)
 }
 
 int
-addr_stob(struct sockaddr *sa, u_short *bits)
+addr_stob(const struct sockaddr *sa, uint16_t *bits)
 {
 	struct sockaddr_in *sin;
 	int i, j, len;
-	u_short n;
+	uint16_t n;
 	u_char *p;
 	
 	sin = (struct sockaddr_in *)sa;
@@ -406,14 +433,14 @@ addr_stob(struct sockaddr *sa, u_short *bits)
 }
 	
 int
-addr_btom(u_short bits, void *mask, size_t size)
+addr_btom(uint16_t bits, void *mask, size_t size)
 {
 	int net, host;
 	u_char *p;
 
 	if (size == IP_ADDR_LEN) {
 		assert(bits <= IP_ADDR_BITS);
-		*(u_int32_t *)mask = bits ?
+		*(uint32_t *)mask = bits ?
 		    htonl(~0 << (IP_ADDR_BITS - bits)) : 0;
 	} else {
 		assert(size * 8 >= bits);
@@ -432,9 +459,9 @@ addr_btom(u_short bits, void *mask, size_t size)
 }
 
 int
-addr_mtob(void *mask, size_t size, u_short *bits)
+addr_mtob(const void *mask, size_t size, uint16_t *bits)
 {
-	u_short n;
+	uint16_t n;
 	u_char *p;
 	int i, j;
 
