@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: arp-none.c,v 1.1.1.1 2001/10/11 04:14:55 dugsong Exp $
+ * $Id: arp-none.c,v 1.4 2002/01/20 21:23:27 dugsong Exp $
  */
 
 #include "config.h"
@@ -19,41 +19,40 @@
 arp_t *
 arp_open(void)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (NULL);
 }
 
 int
-arp_add(arp_t *a, struct addr *pa, struct addr *ha)
+arp_add(arp_t *a, const struct addr *pa, const struct addr *ha)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
 int
-arp_delete(arp_t *a, struct addr *pa)
+arp_delete(arp_t *a, const struct addr *pa)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
 int
-arp_get(arp_t *a, struct addr *pa, struct addr *ha)
+arp_get(arp_t *a, const struct addr *pa, struct addr *ha)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
 int
 arp_loop(arp_t *a, arp_handler callback, void *arg)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 
-int
+arp_t *
 arp_close(arp_t *a)
 {
-	errno = EOPNOTSUPP;
-	return (-1);
+	return (NULL);
 }
