@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: eth-linux.c,v 1.3 2001/12/11 22:41:09 dugsong Exp $
+ * $Id: eth-linux.c,v 1.6 2002/01/20 21:23:27 dugsong Exp $
  */
 
 #include "config.h"
@@ -40,7 +40,7 @@ struct eth_handle {
 };
 
 eth_t *
-eth_open(char *device)
+eth_open(const char *device)
 {
 	eth_t *e;
 	int n;
@@ -48,50 +48,44 @@ eth_open(char *device)
 	if ((e = calloc(1, sizeof(*e))) == NULL)
 		return (NULL);
 	
-	if ((e->fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) {
-		free(e);
-		return (NULL);
-	}
+	if ((e->fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0)
+		return (eth_close(e));
 #ifdef SO_BROADCAST
 	n = 1;
-	if (setsockopt(e->fd, SOL_SOCKET, SO_BROADCAST, &n, sizeof(n)) < 0) {
-		eth_close(e);
-		return (NULL);
-	}
+	if (setsockopt(e->fd, SOL_SOCKET, SO_BROADCAST, &n, sizeof(n)) < 0)
+		return (eth_close(e));
 #endif
 	strlcpy(e->ifr.ifr_name, device, sizeof(e->ifr.ifr_name));
 	
-	if (ioctl(e->fd, SIOCGIFINDEX, &e->ifr) < 0) {
-		eth_close(e);
-		return (NULL);
-	}
+	if (ioctl(e->fd, SIOCGIFINDEX, &e->ifr) < 0)
+		return (eth_close(e));
+
 	e->sll.sll_family = AF_PACKET;
 	e->sll.sll_ifindex = e->ifr.ifr_ifindex;
 	
 	return (e);
 }
 
-ssize_t
+size_t
 eth_send(eth_t *e, const void *buf, size_t len)
 {
 	struct eth_hdr *eth = (struct eth_hdr *)buf;
 	
 	e->sll.sll_protocol = eth->eth_type;
 
-	return (sendto(e->fd, buf, len, 0, (struct sockaddr *)&e->sll,
+	return ((ssize_t)sendto(e->fd, buf, len, 0, (struct sockaddr *)&e->sll,
 	    sizeof(e->sll)));
 }
 
-int
+eth_t *
 eth_close(eth_t *e)
 {
 	assert(e != NULL);
 
-	if (close(e->fd) < 0)
-		return (-1);
-	
+	if (e->fd > 0)
+		close(e->fd);
 	free(e);
-	return (0);
+	return (NULL);
 }
 
 int
@@ -110,7 +104,7 @@ eth_get(eth_t *e, eth_addr_t *ea)
 }
 
 int
-eth_set(eth_t *e, eth_addr_t *ea)
+eth_set(eth_t *e, const eth_addr_t *ea)
 {
 	struct addr ha;
 
