@@ -4,7 +4,7 @@
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  * Copyright (c) 1999 Masaki Hirabaru <masaki@merit.edu>
  * 
- * $Id: route-bsd.c,v 1.9 2001/12/26 16:59:54 dugsong Exp $
+ * $Id: route-bsd.c,v 1.12 2002/01/20 21:23:28 dugsong Exp $
  */
 
 #include "config.h"
@@ -165,23 +165,18 @@ route_open(void)
 {
 	route_t *r;
 	
-	if ((r = malloc(sizeof(*r))) == NULL)
+	if ((r = calloc(1, sizeof(*r))) == NULL)
 		return (NULL);
 
 #ifdef HAVE_STREAMS_ROUTE
-	if ((r->fd = open("/dev/route", O_RDWR, 0)) < 0) {
+	if ((r->fd = open("/dev/route", O_RDWR, 0)) < 0)
 #else
-	if ((r->fd = socket(PF_ROUTE, SOCK_RAW, AF_INET)) < 0) {
+	if ((r->fd = socket(PF_ROUTE, SOCK_RAW, AF_INET)) < 0)
 #endif
-		free(r);
-		return (NULL);
-	}
+		return (route_close(r));
 #ifdef HAVE_STREAMS_MIB2
-	if ((r->ip_fd = open(IP_DEV_NAME, O_RDWR)) < 0) {
-		close(r->fd);
-		free(r);
-		return (NULL);
-	}
+	if ((r->ip_fd = open(IP_DEV_NAME, O_RDWR)) < 0)
+		return (route_close(r));
 #endif
 	r->pid = getpid();
 	r->seq = 0;
@@ -190,20 +185,21 @@ route_open(void)
 }
 
 int
-route_add(route_t *r, struct addr *dst, struct addr *gw)
+route_add(route_t *r, const struct addr *dst, const struct addr *gw)
 {
 	u_char buf[BUFSIZ];
 
 	assert(dst != NULL && gw != NULL);
 
-	if (route_msg(r, RTM_ADD, buf, sizeof(buf), dst, gw) < 0)
+	if (route_msg(r, RTM_ADD, buf, sizeof(buf), (struct addr *)dst,
+	    (struct addr *)gw) < 0)
 		return (-1);
 	
 	return (0);
 }
 
 int
-route_delete(route_t *r, struct addr *dst)
+route_delete(route_t *r, const struct addr *dst)
 {
 	struct addr gw;
 	u_char buf[BUFSIZ];
@@ -213,20 +209,22 @@ route_delete(route_t *r, struct addr *dst)
 	if (route_get(r, dst, &gw) < 0)
 		return (-1);
 	
-	if (route_msg(r, RTM_DELETE, buf, sizeof(buf), dst, &gw) < 0)
+	if (route_msg(r, RTM_DELETE, buf, sizeof(buf),
+	    (struct addr *)dst, &gw) < 0)
 		return (-1);
 	
 	return (0);
 }
 
 int
-route_get(route_t *r, struct addr *dst, struct addr *gw)
+route_get(route_t *r, const struct addr *dst, struct addr *gw)
 {
 	u_char buf[BUFSIZ];
 	
 	assert(dst != NULL && gw != NULL);
 	
-	if (route_msg(r, RTM_GET, buf, sizeof(buf), dst, gw) < 0)
+	if (route_msg(r, RTM_GET, buf, sizeof(buf),
+	    (struct addr *)dst, gw) < 0)
 		return (-1);
 	
 	return (0);
@@ -428,23 +426,22 @@ route_loop(route_t *r, route_handler callback, void *arg)
 int
 route_loop(route_t *r, route_handler callback, void *arg)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 #endif
 
-int
+route_t *
 route_close(route_t *r)
 {
 	assert(r != NULL);
 
-	if (
 #ifdef HAVE_STREAMS_MIB2
-		close(r->ip_fd) < 0 ||
+	if (r->ip_fd > 0)
+		close(r->ip_fd);
 #endif
-		close(r->fd) < 0)
-		return (-1);
-	
+	if (r->fd > 0)
+		close(r->fd);
 	free(r);
-	return (0);
+	return (NULL);
 }
