@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: route-linux.c,v 1.6 2001/12/29 21:06:41 dugsong Exp $
+ * $Id: route-linux.c,v 1.10 2002/01/22 21:37:58 dugsong Exp $
  */
 
 #include "config.h"
@@ -45,27 +45,23 @@ route_open(void)
 	if ((r = calloc(1, sizeof(*r))) == NULL)
 		return (NULL);
 
-	if ((r->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-		free(r);
-		return (NULL);
-	}
-	if ((r->nlfd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE)) < 0) {
-		close(r->fd);
-		free(r);
-		return (NULL);
-	}
+	if ((r->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+		return (route_close(r));
+	
+	if ((r->nlfd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE)) < 0)
+		return (route_close(r));
+	
 	memset(&snl, 0, sizeof(snl));
 	snl.nl_family = AF_NETLINK;
 
-	if (bind(r->nlfd, (struct sockaddr *)&snl, sizeof(snl)) < 0) {
-		route_close(r);
-		return (NULL);
-	}
+	if (bind(r->nlfd, (struct sockaddr *)&snl, sizeof(snl)) < 0)
+		return (route_close(r));
+	
 	return (r);
 }
 
 int
-route_add(route_t *r, struct addr *dst, struct addr *gw)
+route_add(route_t *r, const struct addr *dst, const struct addr *gw)
 {
 	struct rtentry rt;
 
@@ -89,7 +85,7 @@ route_add(route_t *r, struct addr *dst, struct addr *gw)
 }
 
 int
-route_delete(route_t *r, struct addr *dst)
+route_delete(route_t *r, const struct addr *dst)
 {
 	struct rtentry rt;
 
@@ -112,7 +108,7 @@ route_delete(route_t *r, struct addr *dst)
 }
 
 int
-route_get(route_t *r, struct addr *dst, struct addr *gw)
+route_get(route_t *r, const struct addr *dst, struct addr *gw)
 {
 	static int seq;
 	struct nlmsghdr *nmsg;
@@ -204,7 +200,7 @@ route_loop(route_t *r, route_handler callback, void *arg)
 	char buf[BUFSIZ], ifbuf[16];
 	int i, iflags, refcnt, use, metric, mss, win, irtt, ret;
 	struct addr dst, gw;
-	u_int32_t mask;
+	uint32_t mask;
 
 	dst.addr_type = gw.addr_type = ADDR_TYPE_IP;
 	dst.addr_bits = gw.addr_bits = IP_ADDR_BITS;
@@ -242,14 +238,15 @@ route_loop(route_t *r, route_handler callback, void *arg)
 	return (ret);
 }
 
-int
+route_t *
 route_close(route_t *r)
 {
 	assert(r != NULL);
 
-	if (close(r->fd) < 0 || close(r->nlfd) < 0)
-		return (-1);
-	
+	if (r->fd > 0)
+		close(r->fd);
+	if (r->nlfd > 0)
+		close(r->nlfd);
 	free(r);
-	return (0);
+	return (NULL);
 }
