@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: ip.c,v 1.3 2001/12/11 22:41:40 dugsong Exp $
+ * $Id: ip.c,v 1.6 2002/01/07 02:27:05 dugsong Exp $
  */
 
 #include "config.h"
@@ -11,10 +11,10 @@
 #include <sys/types.h>
 
 #include <err.h>
-#include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "dnet.h"
@@ -30,7 +30,7 @@ ip_usage(int die)
 }
 
 static int
-proto_aton(char *string, u_char *proto)
+proto_aton(char *string, uint8_t *proto)
 {
 	struct protoent *pp;
 	long l;
@@ -57,7 +57,7 @@ ip_main(int argc, char *argv[])
 	int c, len;
 	ip_t *i = NULL;
 
-	srand(getpid());
+	srand(time(NULL));
 
 	ip = (struct ip_hdr *)buf;
 	ip->ip_hl = 5;
